""" Usefull tools """

import os

__all__ = ['dump_gcr', 'fits_to_hdf5']

def dump_gcr(g, outfile, particles="all", mode='w-', **kwargs):
    """Dump full cosmic ray distribution.

    Parameters
    ----------
    g : galpropy.Galprop
        The Galprop instance.
    outfile : str
        Output file name including the path. The directory will be created if
        does not exist.
    particles : list[str] or set[str], or "all", optional
        Specify the Particles to dump. (default "all")
    mode : {'w-', 'x', 'w', 'a'}, optional
        w- or x  Create file, fail if exists  (default)
        w        Create file, truncate if exists
        a        Read/write if exists, create otherwise
    kwargs : dict, optional
        Extra arguments to `h5py.File`.

    """
    try:
        import h5py
        import numpy as np
    except ImportError:
        print("To store GCR please install h5py and numpy, nothing dumped")
        return
    outfile = os.path.abspath(outfile)
    os.makedirs(os.path.dirname(outfile), exist_ok=True)
    if particles == "all":
        particles = {p.name for p in g.gcr}

    with h5py.File(outfile, mode, **kwargs) as f:
        dset = f.create_dataset('r', data=g.gcr[0].r())
        dset.attrs['unit'] = 'kpc'
        dset = f.create_dataset('z', data=g.gcr[0].z())
        dset.attrs['unit'] = 'kpc'
        dset = f.create_dataset('Ekin', data=g.GetEkin())
        dset.attrs['unit'] = 'GeV/n'
        for gcr in g.gcr:
            if gcr.name in particles:
                group = f.create_group(gcr.name)
                crdist = gcr.cr_density
                dist = gcr.A_pos() * 1e3 * np.array(crdist.dist).reshape(
                        (crdist.n_rgrid, crdist.n_zgrid, crdist.n_pgrid))
                group.create_dataset('flux', data=dist)
                group.attrs['Z'] = gcr.Z()
                group.attrs['A'] = gcr.A()
                group.attrs['unit'] = r'cm^-2 s^-1 sr^-1 GeV^-1'


def fits_to_hdf5(fitsfilename, hdf5filename, mode='w-', dtype=None, **kwargs):
    try:
        import numpy as np
        import h5py
        from astropy.io import fits
    except ImportError:
        print("To convert GCR fits file to hdf5 format please install h5py "
              "and astropy.")
        return
    outfile = os.path.abspath(hdf5filename)
    os.makedirs(os.path.dirname(outfile), exist_ok=True)
    dtype = np.float64 if dtype is None else dtype
    with h5py.File(outfile, mode, **kwargs) as hdf5:
        with fits.open(fitsfilename) as hdul:
            data = hdul["PRIMARY"].data
            if "NUCLEI" in hdul:
                nuclei = hdul["NUCLEI"].data
            hdf5.attrs["dim"] = data.ndim - 1
            galz = np.asarray(hdul["GAL-Z"].data, dtype)
            energy = np.asarray(hdul["Energy"].data, dtype)
            dset = hdf5.create_dataset('z', data=galz)
            dset.attrs['unit'] = 'kpc'
            dset = hdf5.create_dataset('Ekin', data=energy * 1e-3)
            dset.attrs['unit'] = 'GeV/n'
            if data.ndim == 5:
                galx = np.asarray(hdul["GAL-X"].data, dtype)
                galy = np.asarray(hdul["GAL-Y"].data, dtype)
                dset = hdf5.create_dataset('x', data=galx)
                dset.attrs['unit'] = 'kpc'
                dset = hdf5.create_dataset('y', data=galy)
                dset.attrs['unit'] = 'kpc'
                transpo = (3, 2, 1, 0)
                dimension = "(x, y, z, Ekin/n)"
            if data.ndim == 4:
                galr = np.asarray(hdul["GAL-R"].data, dtype)
                dset = hdf5.create_dataset('r', data=galr)
                dset.attrs['unit'] = 'kpc'
                transpo = (2, 1, 0)
                dimension = "(r, z, Ekin/n)"
            for i in range(len(nuclei)):
                nucname, Z, A, _, _ = nuclei[i]
                group = hdf5.create_group(nucname)
                store = np.transpose(data[i, ...], transpo) / energy**2 * 1e3
                group.create_dataset('flux', data=store)
                group.attrs['Z'] = Z
                group.attrs['A'] = A
                group.attrs['unit'] = r'cm^-2 s^-1 sr^-1 GeV^-1'
                group.attrs['dimention'] = dimension
